'use strict';

var core = require('@agile-ts/core');
var Vue = require('vue');
var utils = require('@agile-ts/utils');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var Vue__default = /*#__PURE__*/_interopDefaultLegacy(Vue);

function bindAgileInstances(deps, agile, vueComponent, observerType) {
    let depsWithoutIndicator = [];
    let depsWithIndicator;
    // Format deps
    if (utils.isValidObject(deps)) {
        depsWithIndicator = formatDepsWithIndicator(deps, observerType);
    }
    else {
        const response = formatDepsWithNoSafeIndicator(deps, observerType);
        depsWithIndicator = response.depsWithIndicator;
        depsWithoutIndicator = response.depsWithoutIndicator;
    }
    // Create Subscription with extracted Observers
    // that have no unique key/name indicator
    // and thus can't be merged into the 'sharedState' property.
    // (Re-render will be enforced via a force update)
    if (depsWithoutIndicator.length > 0) {
        agile.subController.subscribe(vueComponent, depsWithoutIndicator, {
            waitForMount: false,
        });
    }
    // Create Subscription with extracted Observers
    // that have a unique key/name indicator.
    // (Rerender will be enforced via mutating the 'this.$data.sharedState' property)
    if (depsWithIndicator) {
        return agile.subController.subscribe(vueComponent, depsWithIndicator, {
            waitForMount: false,
        }).props;
    }
    return {};
}
/**
 * Extracts the Observers from the specified dependencies
 * which probably have no safe unique indicator key/name.
 *
 * If a unique key/name indicator could be found
 * the extracted Observer is added to the `depsWithIndicator` object
 * and otherwise to the `depsWithoutIndicator` array.
 *
 * What type of Observer is extracted from a dependency,
 * depends on the specified `observerType`.
 * If no `observerType` is specified, the Observers found in the dependency
 * are selected in the following `observerType` order.
 * 1. `output`
 * 2. `value`
 *
 * @internal
 * @param deps - Dependencies in array shape to extract the Observers from.
 * @param observerType - Type of Observer to be extracted.
 */
const formatDepsWithNoSafeIndicator = (deps, observerType) => {
    const depsArray = core.extractRelevantObservers(utils.normalizeArray(deps), observerType);
    const depsWithIndicator = {};
    let depsWithoutIndicator = depsArray.filter((dep) => dep !== undefined);
    // Try to extract a unique key/name identifiers from the extracted Observers
    depsWithoutIndicator = depsWithoutIndicator.filter((dep) => {
        if (dep && dep['key'] != null) {
            depsWithIndicator[dep['key']] = dep;
            return false;
        }
        return true;
    });
    return {
        depsWithIndicator,
        depsWithoutIndicator,
    };
};
/**
 * Extracts the Observers from the specified dependencies
 * which have a unique key/name identifier
 * through the object property key.
 *
 * What type of Observer is extracted from a dependency,
 * depends on the specified `observerType`.
 * If no `observerType` is specified, the Observers found in the dependency
 * are selected in the following `observerType` order.
 * 1. `output`
 * 2. `value`
 *
 * @internal
 * @param deps - Dependencies in object shape to extract the Observers from.
 * @param observerType - Type of Observer to be extracted.
 */
const formatDepsWithIndicator = (deps, observerType) => {
    const depsObject = core.extractRelevantObservers(deps, observerType);
    const depsWithIndicator = {};
    for (const key in depsObject) {
        const observer = depsObject[key];
        if (observer != null)
            depsWithIndicator[key] = observer;
    }
    return depsWithIndicator;
};

const vueIntegration = new core.Integration({
    key: 'vue',
    frameworkInstance: Vue__default['default'],
    updateMethod: (componentInstance, updatedData) => {
        const componentData = componentInstance.$data;
        // Update existing data or if a new one was added set it via Vue.
        // Note: Not merging the 'updateData' object into the 'componentData' object
        // because Vue tracks the local State changes via a Proxy
        // and by merging the two object together, Vue can't detect the changes.
        for (const key of Object.keys(updatedData)) {
            if (Object.prototype.hasOwnProperty.call(componentData, key)) {
                componentData.sharedState[key] = updatedData[key];
            }
            else {
                // https://vuejs.org/v2/guide/reactivity.html
                Vue__default['default'].set(componentData.sharedState, key, updatedData[key]);
            }
        }
    },
    bind: (agile) => {
        // https://vuejs.org/v2/guide/plugins.html
        Vue__default['default'].use({
            install: (vue) => {
                // https://vuejs.org/v2/guide/mixins.html
                vue.mixin({
                    created: function () {
                        // @ts-ignore
                        this.$agile = agile;
                    },
                    methods: {
                        // TODO make 'bindAgileInstances' ('sharedState') more typesafe
                        bindAgileInstances: function (deps) {
                            return {
                                sharedState: {
                                    ...(this?.$data?.sharedState || {}),
                                    ...bindAgileInstances(deps, agile, this),
                                },
                            };
                        },
                        // TODO make 'bindAgileValues' ('sharedState') more typesafe
                        bindAgileValues: function (deps) {
                            return {
                                sharedState: {
                                    ...(this?.$data?.sharedState || {}),
                                    ...bindAgileInstances(deps, agile, this, 'value'),
                                },
                            };
                        },
                        // TODO make 'bindAgileOutputs' ('sharedState') more typesafe
                        bindAgileOutputs: function (deps) {
                            return {
                                sharedState: {
                                    ...(this?.$data?.sharedState || {}),
                                    ...bindAgileInstances(deps, agile, this, 'output'),
                                },
                            };
                        },
                    },
                });
            },
        });
        return Promise.resolve(true);
    },
});
core.Integrations.addInitialIntegration(vueIntegration);
var vueIntegration$1 = vueIntegration;

module.exports = vueIntegration$1;
